@@ -1,10 +1,15 @@
 package Mite::Attribute;
 use Mite::MyMoo;
 
-has default =>
+has class =>
   is            => rw,
-  isa           => Maybe[Str|Ref],
-  predicate     => 'has_default';
+  isa           => Object,
+  weak_ref      => true;
+
+has name =>
+  is            => rw,
+  isa           => Str->where('length($_) > 0'),
+  required      => true;
 
 has init_arg =>
   is            => rw,
@@ -17,6 +22,39 @@ has required =>
   isa           => Bool,
   default       => false;
 
+has is =>
+  is            => rw,
+  isa           => Enum[ ro, rw, rwp, 'lazy', 'bare' ],
+  default       => 'bare';
+
+has [ 'reader', 'writer', 'accessor', 'clearer', 'predicate' ] =>
+  is            => rw,
+  isa           => Str->where('length($_) > 0') | Undef,
+  builder       => true,
+  lazy          => true;
+
+has isa =>
+  is            => 'bare',
+  isa           => Str,
+  reader        => '_isa',
+  init_arg      => 'isa';
+
+has type =>
+  init_arg      => undef,
+  is            => 'lazy',
+  isa           => Object|Undef,
+  builder       => 1;
+
+has default =>
+  is            => rw,
+  isa           => Maybe[Str|Ref],
+  predicate     => 'has_default';
+
+has lazy =>
+  is            => rw,
+  isa           => Bool,
+  default       => false;
+
 has coderef_default_variable =>
   is            => rw,
   isa           => Str,
@@ -27,26 +65,11 @@ has coderef_default_variable =>
       return sprintf '$__%s_DEFAULT__', $self->name;
   };
 
-has is =>
-  is            => rw,
-  isa           => Enum[ ro, rw, rwp, 'bare' ],
-  default       => 'bare';
-
-has name =>
-  is            => rw,
-  isa           => Str->where('length($_) > 0'),
-  required      => true;
-
-has [ 'reader', 'writer', 'accessor', 'clearer', 'predicate' ] =>
-  is            => rw,
-  isa           => Str->where('length($_) > 0') | Undef,
-  builder       => true,
-  lazy          => true;
-
 # Not actually supported yet
 has builder =>
     is            => rw,
     isa           => Str->where('length($_) > 0') | CodeRef | Undef,
+    predicate     => true;
 
 my @method_name_generator = (
     { # public
@@ -73,6 +96,30 @@ sub BUILD {
     croak "Required attribute with no init_arg"
         if $self->required && !defined $self->init_arg;
 
+    if ( $self->is eq 'lazy' ) {
+        $self->lazy( true );
+        $self->builder( 1 ) unless $self->has_builder;
+        $self->is( 'ro' );
+    }
+
+    if ( CodeRef->check( $self->builder ) ) {
+        my $coderef = $self->builder;
+        my $newname = do {
+            my $gen = $method_name_generator[$self->is_private]{builder};
+            local $_ = $self->name;
+            $gen->( $_ );
+        };
+        no strict 'refs';
+        my $classname;
+        if ( $self->class and $classname = $self->class->name ) {
+            *{"$classname\::$newname"} = $coderef;
+            $self->builder( $newname );
+        }
+        else {
+            croak "Could not install builder=>CODEREF as Mite could not determine which class to install it into.";
+        }
+    }
+
     for my $property ( 'reader', 'writer', 'accessor', 'clearer', 'predicate', 'builder' ) {
         my $name = $self->$property;
         if ( defined $name and $name eq 1 ) {
@@ -126,6 +173,21 @@ sub _build_predicate { undef; }
 
 sub _build_clearer { undef; }
 
+sub _build_type {
+    my $self = shift;
+
+    my $isa = $self->_isa
+        or return undef;
+
+    require Type::Utils;
+    my $type = Type::Utils::dwim_type( $isa, fallback => [ 'make_class_type' ] );
+
+    $type->can_be_inlined
+        or croak sprintf 'Type %s cannot be inlined', $type->display_name;
+
+    return $type;
+}
+
 sub has_dataref_default {
     my $self = shift;
 
@@ -164,6 +226,16 @@ sub _empty {
     return ';';
 }
 
+sub _compile_checked_default {
+    my ( $self, $selfvar ) = @_;
+
+    my $default = $self->_compile_default( $selfvar );
+    my $type = $self->type or return $default;
+
+    return sprintf 'do { my $default_value = %s; %s or do { require Carp; Carp::croak(q[Type check failed in default: %s should be %s]) }; $default_value }',
+        $default, $type->inline_check('$default_value'), $self->name, $type->display_name;
+}
+
 sub _compile_default {
     my ( $self, $selfvar ) = @_;
 
@@ -176,50 +248,55 @@ sub _compile_default {
         require B;
         return defined( $self->default ) ? B::perlstring( $self->default ) : 'undef';
     }
+    elsif ( $self->has_builder ) {
+        return sprintf '%s->%s', $selfvar, $self->builder;
+    }
 
     # should never get here
     return 'undef';
 }
 
-sub _compile_required_error {
-    my $self = shift;
-
-    return sprintf 'do { require Carp; Carp::croak("Missing key in contructor: %s") }',
-        $self->init_arg;
-}
-
 sub compile_init {
     my ( $self, $selfvar, $argvar ) = @_;
 
     my $init_arg = $self->init_arg;
 
-    if ( $self->has_default and not defined $init_arg ) {
-        return sprintf '%s->{%s} = %s;',
-            $selfvar, $self->name,
-            $self->_compile_default( $selfvar );
-    }
-
-    if ( $self->has_default ) {
-        return sprintf '%s->{%s} = exists(%s->{%s}) ? delete(%s->{%s}) : %s;',
-            $selfvar, $self->name,
-            $argvar,  $init_arg,
-            $argvar,  $init_arg,
-            $self->_compile_default( $selfvar );
-    }
-
+    my $code = '';
     if ( defined $init_arg ) {
-        if ( $self->required ) {
-            return sprintf '%s->{%s} = exists(%s->{%s}) ? delete(%s->{%s}) : %s;',
-                $selfvar, $self->name,
-                $argvar,  $init_arg,
-                $argvar,  $init_arg,
-                $self->_compile_required_error;
+        $code .= sprintf 'if ( exists(%s->{q[%s]}) ) { ',
+            $argvar, $init_arg;
+        if ( my $type = $self->type ) {
+            local $Type::Tiny::AvoidCallbacks = 1;
+            $code .= sprintf '%s or do { require Carp; Carp::croak(q[Type check failed in constructor: %s should be %s]) }; ',
+                $type->inline_check(sprintf '%s->{q[%s]}', $argvar, $init_arg),
+                $self->init_arg,
+                $type->display_name;
         }
-        return sprintf '%s->{%s} = delete(%s->{%s}) if exists(%s->{%s});',
-            $selfvar, $self->name,
-            $argvar,  $init_arg,
-            $argvar,  $init_arg;
+        $code .= sprintf '%s->{q[%s]} = delete %s->{q[%s]}; ',
+            $selfvar, $self->name, $argvar, $init_arg;
+        $code .= ' }';
     }
+
+    if ( $self->has_default || $self->has_builder
+    and not $self->lazy ) {
+        if ( $code ) {
+            $code .= ' else { ';
+        }
+        else {
+            $code .= 'do { ';
+        }
+        $code .= sprintf '%s->{q[%s]} = %s; ',
+            $selfvar, $self->name, $self->_compile_checked_default( $selfvar );
+        $code .= ' }';
+    }
+    elsif ( defined $init_arg
+    and $self->required
+    and not ($self->has_default || $self->has_builder) ) {
+        $code .= sprintf ' else { require Carp; Carp::croak("Missing key in constructor: %s") }',
+            $init_arg;
+    }
+
+    return $code;
 }
 
 sub compile {
@@ -237,19 +314,44 @@ CODE
 
 my %code_template = (
     reader => sub {
-        my $slot_name = shift->name;
-        sprintf '@_ > 1 ? require Carp && Carp::croak("%s is a read-only attribute of @{[ref $_[0]]}") : $_[0]->{ q[%s] }',
-            $slot_name, $slot_name;
+        my $self = shift;
+        my $slot_name = $self->name;
+        my $code = sprintf '$_[0]->{q[%s]}', $slot_name;
+        if ( $self->lazy ) {
+            $code = sprintf '( exists($_[0]{q[%s]}) ? $_[0]{q[%s]} : ( $_[0]{q[%s]} = %s ) )',
+                $slot_name, $slot_name, $slot_name, $self->_compile_checked_default( '$_[0]' );
+        }
+        $code = sprintf '@_ > 1 ? require Carp && Carp::croak("%s is a read-only attribute of @{[ref $_[0]]}") : %s',
+            $slot_name, $code;
+        return $code;
     },
     writer => sub {
-        my $slot_name = shift->name;
-        sprintf '$_[0]->{ q[%s] } = $_[1];',
-            $slot_name;
+        my $self = shift;
+        my $slot_name = $self->name;
+        my $code = sprintf '$_[0]->{ q[%s] } = $_[1];', $slot_name;
+        if ( my $type = $self->type ) {
+            local $Type::Tiny::AvoidCallbacks = 1;
+            $code = sprintf '%s or do { require Carp; Carp::croak(q[Type check failed in writer: value should be %s]) }; %s',
+                $type->inline_check('$_[1]'), $type->display_name, $code;
+        }
+        return $code;
     },
     accessor => sub {
-        my $slot_name = shift->name;
-        sprintf '@_ > 1 ? $_[0]->{ q[%s] } = $_[1] : $_[0]->{ q[%s] }',
-            $slot_name, $slot_name;
+        my $self = shift;
+        my $slot_name = $self->name;
+        my $code = sprintf '$_[0]->{q[%s]}', $slot_name;
+        if ( $self->lazy ) {
+            $code = sprintf '( exists($_[0]{q[%s]}) ? $_[0]{q[%s]} : ( $_[0]{q[%s]} = %s ) )',
+                $slot_name, $slot_name, $slot_name, $self->_compile_checked_default( '$_[0]' );
+        }
+        $code = sprintf '@_ > 1 ? ( $_[0]->{ q[%s] } = $_[1] ) : %s',
+            $slot_name, $code;
+        if ( my $type = $self->type ) {
+            local $Type::Tiny::AvoidCallbacks = 1;
+            $code = sprintf '@_==1 or %s or do { require Carp; Carp::croak(q[Type check failed in accessor: value should be %s]) }; %s',
+                $type->inline_check('$_[1]'), $type->display_name, $code;
+        }
+        return $code;
     },
     clearer => sub {
         my $slot_name = shift->name;
@@ -275,9 +377,31 @@ sub _compile_xs {
         predicate => 'exists_predicates',
     );
 
+    my %use_xs = (
+        'reader' => 1,
+        'writer' => 1,
+        'accessor' => 1,
+        'predicate' => 1,
+    );
+    my %use_pp = (
+        'clearer' => 1,
+    );
+
+    if ( $self->type ) {
+        delete $use_xs{writer};
+        delete $use_xs{accessor};
+        $use_pp{writer} = $use_pp{accessor} = 1;
+    }
+
+    if ( $self->lazy ) {
+        delete $use_xs{reader};
+        delete $use_xs{accessor};
+        $use_pp{reader} = $use_pp{accessor} = 1;
+    }
+
     my $xs = 0;
     my $code = "Class::XSAccessor->import(\n";
-    for my $property ( 'reader', 'writer', 'accessor', 'predicate' ) {
+    for my $property ( sort keys %use_xs ) {
         my $method_name = $self->$property;
         next unless defined $method_name;
         my $option = $options{$property};
@@ -287,10 +411,11 @@ sub _compile_xs {
     $code .= ");\n";
     $code = '' unless $xs;
 
-    # Class::XSAccessor doesn't have clearers
-    if ( my $clearer = $self->clearer ) {
+    for my $property ( sort keys %use_pp ) {
+        my $method_name = $self->$property;
+        next unless defined $method_name;
         $code .= sprintf '*%s = sub { %s };' . "\n",
-            $clearer, $code_template{clearer}->($self);
+            $method_name, $code_template{$property}->($self);
     }
 
     return $code;
