@@ -6,7 +6,7 @@ package Mite::Shim;
 use strict;
 use warnings;
 
-use version 0.77; our $VERSION = qv("v0.0.3");
+use version 0.77; our $VERSION = qv("v0.0.4");
 
 sub _is_compiling {
     return $ENV{MITE_COMPILE} ? 1 : 0;
@@ -50,9 +50,14 @@ sub import {
             my %args = @_;
 
             my $default = $args{default};
-            return unless ref $default eq 'CODE';
+            if ( ref $default eq 'CODE' ) {
+                ${$caller .'::__'.$name.'_DEFAULT__'} = $default;
+            }
 
-            ${$caller .'::__'.$name.'_DEFAULT__'} = $default;
+            my $builder = $args{builder};
+            if ( ref $builder eq 'CODE' ) {
+                *{"$caller\::_build_$name"} = $builder;
+            }
 
             return;
         };
